@@ -25,8 +25,8 @@
 #include <iostream>
 #include <QString>
 #include <QStringList>
-#include <QtGlobal>
 #include "remote_control.h"
+#include "qtgui/dockrxopt.h"
 
 #define DEFAULT_RC_PORT            7356
 #define DEFAULT_RC_ALLOWED_HOSTS   "::ffff:127.0.0.1"
@@ -42,6 +42,8 @@ RemoteControl::RemoteControl(QObject *parent) :
     rc_mode = 0;
     rc_passband_lo = 0;
     rc_passband_hi = 0;
+    rc_program_id = "0000";
+    rds_status = false;
     signal_level = -200.0;
     squelch_level = -150.0;
     audio_recorder_status = false;
@@ -53,15 +55,7 @@ RemoteControl::RemoteControl(QObject *parent) :
 
     rc_socket = 0;
 
-#if QT_VERSION < 0x050900
-    // Disable proxy setting detected by Qt
-    // Workaround for https://bugreports.qt.io/browse/QTBUG-58374
-    // Fix: https://codereview.qt-project.org/#/c/186124/
-    rc_server.setProxy(QNetworkProxy::NoProxy);
-#endif
-
     connect(&rc_server, SIGNAL(newConnection()), this, SLOT(acceptConnection()));
-
 }
 
 RemoteControl::~RemoteControl()
@@ -233,6 +227,8 @@ void RemoteControl::startRead()
         answer = cmd_get_split_vfo();
     else if (cmd == "S")
         answer = cmd_set_split_vfo();
+    else if (cmd == "p")
+        answer = cmd_get_param(cmdlist);
     else if (cmd == "_")
         answer = cmd_get_info();
     else if (cmd == "AOS")
@@ -390,6 +386,18 @@ bool RemoteControl::setGain(QString name, double gain)
     return false;
 }
 
+/*! \brief Set RDS program identification (from RDS parser). */
+void RemoteControl::rdsPI(QString program_id)
+{
+    rc_program_id = program_id;
+}
+
+/*! \brief Set RDS status (from RDS dock). */
+void RemoteControl::setRDSstatus(bool enabled)
+{
+    rds_status = enabled;
+    rc_program_id = "0000";
+}
 
 /*! \brief Convert mode string to enum (DockRxOpt::rxopt_mode_idx)
  *  \param mode The Hamlib rigctld compatible mode string
@@ -404,65 +412,64 @@ int RemoteControl::modeStrToInt(QString mode_str)
 
     if (mode_str.compare("OFF", Qt::CaseInsensitive) == 0)
     {
-        mode_int = 0;
+        mode_int = DockRxOpt::MODE_OFF;
     }
     else if (mode_str.compare("RAW", Qt::CaseInsensitive) == 0)
     {
-        mode_int = 1;
+        mode_int = DockRxOpt::MODE_RAW;
     }
     else if (mode_str.compare("AM", Qt::CaseInsensitive) == 0)
     {
-        mode_int = 2;
+        mode_int = DockRxOpt::MODE_AM;
     }
-    else if (mode_str.compare("FM", Qt::CaseInsensitive) == 0)
+    else if (mode_str.compare("AMS", Qt::CaseInsensitive) == 0)
     {
-        mode_int = 3;
-    }
-    else if (mode_str.compare("WFM", Qt::CaseInsensitive) == 0)
-    {
-        mode_int = 4;
-    }
-    else if (mode_str.compare("WFM_ST", Qt::CaseInsensitive) == 0)
-    {
-        mode_int = 5;
+        mode_int = DockRxOpt::MODE_AM_SYNC;
     }
     else if (mode_str.compare("LSB", Qt::CaseInsensitive) == 0)
     {
-        mode_int = 6;
+        mode_int = DockRxOpt::MODE_LSB;
     }
     else if (mode_str.compare("USB", Qt::CaseInsensitive) == 0)
     {
-        mode_int = 7;
-    }
-    else if (mode_str.compare("CW", Qt::CaseInsensitive) == 0)
-    {
-        mode_int = 9;
-        hamlib_compatible = true;
+        mode_int = DockRxOpt::MODE_USB;
     }
     else if (mode_str.compare("CWL", Qt::CaseInsensitive) == 0)
     {
-        mode_int = 8;
+        mode_int = DockRxOpt::MODE_CWL;
         hamlib_compatible = false;
     }
-    else if (mode_str.compare("CWR", Qt::CaseInsensitive) == 0)
+    else if (mode_str.compare("CWR", Qt::CaseInsensitive) == 0)  // "CWR" : "CWL"
     {
-        mode_int = 8;
+        mode_int = DockRxOpt::MODE_CWL;
         hamlib_compatible = true;
     }
     else if (mode_str.compare("CWU", Qt::CaseInsensitive) == 0)
     {
-        mode_int = 9;
+        mode_int = DockRxOpt::MODE_CWU;
         hamlib_compatible = false;
     }
+    else if (mode_str.compare("CW", Qt::CaseInsensitive) == 0)  // "CW" : "CWU"
+    {
+        mode_int = DockRxOpt::MODE_CWU;
+        hamlib_compatible = true;
+    }
+    else if (mode_str.compare("FM", Qt::CaseInsensitive) == 0)
+    {
+        mode_int = DockRxOpt::MODE_NFM;
+    }
+    else if (mode_str.compare("WFM", Qt::CaseInsensitive) == 0)
+    {
+        mode_int = DockRxOpt::MODE_WFM_MONO;
+    }
+    else if (mode_str.compare("WFM_ST", Qt::CaseInsensitive) == 0)
+    {
+        mode_int = DockRxOpt::MODE_WFM_STEREO;
+    }
     else if (mode_str.compare("WFM_ST_OIRT", Qt::CaseInsensitive) == 0)
     {
-        mode_int = 10;
+        mode_int = DockRxOpt::MODE_WFM_STEREO_OIRT;
     }
-    else if (mode_str.compare("AMS", Qt::CaseInsensitive) == 0)
-    {
-        mode_int = 11;
-    }
-
     return mode_int;
 }
 
@@ -476,52 +483,52 @@ QString RemoteControl::intToModeStr(int mode)
 
     switch (mode)
     {
-    case 0:
+    case DockRxOpt::MODE_OFF:
         mode_str = "OFF";
         break;
 
-    case 1:
+    case DockRxOpt::MODE_RAW:
         mode_str = "RAW";
         break;
 
-    case 2:
+    case DockRxOpt::MODE_AM:
         mode_str = "AM";
         break;
 
-    case 3:
-        mode_str = "FM";
+    case DockRxOpt::MODE_AM_SYNC:
+        mode_str = "AMS";
         break;
 
-    case 4:
-        mode_str = "WFM";
-        break;
-
-    case 5:
-        mode_str = "WFM_ST";
-        break;
-
-    case 6:
+    case DockRxOpt::MODE_LSB:
         mode_str = "LSB";
         break;
 
-    case 7:
+    case DockRxOpt::MODE_USB:
         mode_str = "USB";
         break;
 
-    case 8:
+    case DockRxOpt::MODE_CWL:
         mode_str = (hamlib_compatible) ? "CWR" : "CWL";
         break;
 
-    case 9:
+    case DockRxOpt::MODE_CWU:
         mode_str = (hamlib_compatible) ? "CW" : "CWU";
         break;
 
-    case 10:
-        mode_str = "WFM_ST_OIRT";
+    case DockRxOpt::MODE_NFM:
+        mode_str = "FM";
         break;
 
-    case 11:
-        mode_str = "AMS";
+    case DockRxOpt::MODE_WFM_MONO:
+        mode_str = "WFM";
+        break;
+
+    case DockRxOpt::MODE_WFM_STEREO:
+        mode_str = "WFM_ST";
+        break;
+
+    case DockRxOpt::MODE_WFM_STEREO_OIRT:
+        mode_str = "WFM_ST_OIRT";
         break;
 
     default:
@@ -568,7 +575,7 @@ QString RemoteControl::cmd_set_mode(QStringList cmdlist)
     QString cmd_arg = cmdlist.value(1, "");
 
     if (cmd_arg == "?")
-        answer = QString("OFF RAW AM AMS FM WFM WFM_ST WFM_ST_OIRT LSB USB CW CWU CWR CWL\n");
+        answer = QString("OFF RAW AM AMS LSB USB CWL CWR CWU CW FM WFM WFM_ST WFM_ST_OIRT\n");
     else
     {
         int mode = modeStrToInt(cmd_arg);
@@ -691,11 +698,13 @@ QString RemoteControl::cmd_get_func(QStringList cmdlist)
     QString func = cmdlist.value(1, "");
 
     if (func == "?")
-        answer = QString("RECORD DSP\n");
+        answer = QString("RECORD DSP RDS\n");
     else if (func.compare("RECORD", Qt::CaseInsensitive) == 0)
         answer = QString("%1\n").arg(audio_recorder_status);
     else if (func.compare("DSP", Qt::CaseInsensitive) == 0)
         answer = QString("%1\n").arg(receiver_running);
+    else if (func.compare("RDS", Qt::CaseInsensitive) == 0)
+        answer = QString("%1\n").arg(rds_status);
     else
         answer = QString("RPRT 1\n");
 
@@ -712,7 +721,7 @@ QString RemoteControl::cmd_set_func(QStringList cmdlist)
 
     if (func == "?")
     {
-        answer = QString("RECORD DSP\n");
+        answer = QString("RECORD DSP RDS\n");
     }
     else if ((func.compare("RECORD", Qt::CaseInsensitive) == 0) && ok)
     {
@@ -739,6 +748,15 @@ QString RemoteControl::cmd_set_func(QStringList cmdlist)
 
         answer = QString("RPRT 0\n");
     }
+    else if ((func.compare("RDS", Qt::CaseInsensitive) == 0) && ok)
+    {
+        if (status)
+            emit newRDSmode(true);
+        else
+            emit newRDSmode(false);
+
+        answer = QString("RPRT 0\n");
+    }
     else
     {
         answer = QString("RPRT 1\n");
@@ -747,6 +765,22 @@ QString RemoteControl::cmd_set_func(QStringList cmdlist)
     return answer;
 }
 
+/* Get parameter */
+QString RemoteControl::cmd_get_param(QStringList cmdlist)
+{
+    QString answer;
+    QString func = cmdlist.value(1, "");
+
+    if (func == "?")
+        answer = QString("RDS_PI\n");
+    else if (func.compare("RDS_PI", Qt::CaseInsensitive) == 0)
+        answer = QString("%1\n").arg(rc_program_id);
+    else
+        answer = QString("RPRT 1\n");
+
+    return answer;
+}
+
 /* Get current 'VFO' (fake, only for hamlib) */
 QString RemoteControl::cmd_get_vfo() const
 {
@@ -857,7 +891,7 @@ QString RemoteControl::cmd_dump_state() const
         "0.000000 10000000000.000000 0x2ef -1 -1 0x1 0x0\n"
         /* End of RX frequency ranges. */
         "0 0 0 0 0 0 0\n"
-        /* End of TX frequency ranges. The Gqrx is reciver only. */
+        /* End of TX frequency ranges. The Gqrx is receiver only. */
         "0 0 0 0 0 0 0\n"
         /* Tuning steps: modes, tuning_step */
         "0xef 1\n"
